import 'package:flutter/material.dart';

typedef ColoredWidgetBuilder = Widget Function(BuildContext context, Color color);

class BlinkingWidget extends StatefulWidget {
  /// Creates a [BlinkingWidget] widget
  const BlinkingWidget({
    Key? key,
    this.beginColor,
    this.endColor,
    this.duration,
    this.repeats,
    required this.builder,
    this.disabled = false,
  }) : super(key: key);

  final ColoredWidgetBuilder builder;

  /// Duration of blinking animation, default is 1 second.
  final Duration? duration;

  /// Times to repeat color animation. [null] is infinite, 0 is without animation. 
  final int? repeats;

  /// Starting color, default value is [Colors.black].
  final Color? beginColor;

  /// End color, default value is [Colors.transparent].
  final Color? endColor;

  // Disable animation. Build widget only once with beginColor.
  final bool disabled;

  @override
  BlinkingWidgetState createState() => BlinkingWidgetState();
}

class BlinkingWidgetState extends State<BlinkingWidget> with TickerProviderStateMixin {
  AnimationController? controller;
  late Animation<Color?> colorAnimation;
  int repeatCounter = 0;
  late Duration duration;
  late Color beginColor;
  late bool disabled;

  @override
  void initState() {
    super.initState();
    _initializeController();
  }

  @override
  void didUpdateWidget(covariant BlinkingWidget oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (widget.disabled != oldWidget.disabled ||
        widget.repeats != oldWidget.repeats ||
        widget.endColor != oldWidget.endColor ||
        widget.duration != oldWidget.duration) {
      repeatCounter = 0;
      _initializeController();
    }
  }

  void _initializeController() {
    print('_initializeController');

    // Default start color
    beginColor = widget.beginColor ?? Colors.black;

    final repeats = widget.repeats;

    disabled = widget.disabled;
    if (disabled == true || repeats == 0) {
      controller?.stop();
      return;
    }

    // Default duration
    duration = widget.duration ?? Duration(milliseconds: 1000);

    // Default end color
    final endColor = widget.endColor ?? Colors.transparent;

    // Initialize controller and animation
    controller?.dispose();
    controller = AnimationController(vsync: this, duration: duration);
    colorAnimation = ColorTween(
      begin: beginColor,
      end: endColor,
    ).animate(
      CurvedAnimation(
        parent: controller!,
        curve: Curves.linear,
      ),
    );

    controller!.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        repeatCounter++;
        controller!.reverse().then((value) {
          if (disabled || (repeats != null && repeatCounter >= repeats)) {
            controller!.stop();
          }
        });
      } else if (status == AnimationStatus.dismissed) {
        controller!.forward();
      }
      setState(() {});
    });

    controller!.forward();
  }

  @override
  void dispose() {
    controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (controller == null || disabled == true) {
      return widget.builder(context, beginColor);
    } else {
      return AnimatedBuilder(
        animation: controller!,
        builder: (context, child) => widget.builder(context, colorAnimation.value!),
      );
    }
  }
}
